﻿//-----------------------------------------------------------------------
// <copyright file="Program.cs" company="BME MIT" />
//-----------------------------------------------------------------------
/*
 * Name:    IRF.ConsoleGuessGame.Program
 * Author:  Zoltan Micskei, BME MIT
 * Date:    2014.04.08.
 * Desc:    ConsoleGuessGame sample application for demonstrating manageable applications 
 *          see: http://www.inf.mit.bme.hu/edu/bsc/irf
 *          
 *          Startup class for the application. Checks the parameters, instantiates the GuessGame component,
 *          then manages input and output for the console
 *          
 * Param:   name of the game session
 */

namespace IRF.ConsoleGuessGame
{   
    using System;
    using System.Diagnostics;
    using System.Globalization;    
    using System.Management.Instrumentation;
    using System.Reflection;
    using System.Resources;

    using IRF.ConsoleGuessGame.Properties;

    using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
    using Microsoft.Practices.EnterpriseLibrary.Logging;   

    /// <summary>
    /// Contains the entry point of the application, and handles console input and output.
    /// </summary>
    public static class Program
    {
        /// <summary>
        /// Gets the default Enterprise Library Logging Application Block LogWriter used by the application.
        /// </summary>
        internal static LogWriter Logger { get; private set; }

        /// <summary>
        /// Gets the default Enterprise Library Logging Application Block TraceManager used by the application.
        /// </summary>
        internal static TraceManager Tracer { get; private set; }
        
        /// <summary>
        /// Gets the GuessGame instance for the current application.
        /// </summary>
        internal static GuessGame Game { get; private set; }        

        /// <summary>
        /// Entry point for the application.
        /// </summary>
        /// <param name="args">one sessionName string parameter</param>
        public static void Main(string[] args)
        {
            // initialize EntLib logging
            try
            {                
                Program.Logger = (new LogWriterFactory()).Create();
                Program.Tracer = new TraceManager(Program.Logger);                
            }
            catch (Exception ex)
            {
                Console.WriteLine(Resources.LoggingInitFailedUIMessage, ex.Message);
                
                // depends on the application whether it is a fatal error or not
                return;
            }
                        
            // check command line arguments
            if (args == null || args.Length != 1)
            {
                string message = String.Format(
                    CultureInfo.InvariantCulture, 
                    Resources.InvalidStartupParametersLogMessage, 
                    args == null ? "null" : String.Join(",", args));
                
                Program.Log(message, LogEventIds.InvalidStartupParams, TraceEventType.Critical);

                Console.WriteLine(Resources.ImproperParametersUIMessage);
                Console.WriteLine(Resources.ProgramUsageUIMessage, Process.GetCurrentProcess().ProcessName);
                
                return;
            }

            // check whether the required performance counters exists
            if (!PerformanceCounterCategory.Exists(Counters.CategoryName))
            {
                Console.WriteLine(Resources.PerformanceCountersNotExistUIMessage);
                return;
            }
             
            // initialize WMI provider for this session
            GuessGameProvider provider = null;
            try
            {
                provider = new GuessGameProvider(args[0]);                

                // use the Publish/Revoke model instead of the Callback
                InstrumentationManager.Publish(provider);                

                Program.Log(Resources.ProviderCreatedLogMessage, LogEventIds.ProviderCreated, TraceEventType.Verbose);
            }
            catch (Exception ex)
            {
                string message = String.Format(CultureInfo.InvariantCulture, Resources.ProviderNotCreatedLogMessage, ex.Message);
                Program.Log(message, LogEventIds.ProviderNotCreated, TraceEventType.Critical);
                Console.WriteLine(Resources.ProviderNotCreatedUIMessage, ex.Message);
                return;
            }

            // create a new game instance and start a new round
            try
            {
                Game = new GuessGame(args[0]);
            }
            catch (Exception ex)
            {
                string message = String.Format(CultureInfo.InvariantCulture, Resources.GameNotCreatedLogMessage, ex.Message);
                Program.Log(message, LogEventIds.GameNotCreated, TraceEventType.Critical);
                Console.WriteLine(Resources.GameNotCreatedUIMessage, ex.Message);
                return;
            }

            // all objects are initialized, guessing can begin
            Program.Log(Resources.SuccessfullInitializationLogMessage, LogEventIds.SuccessfullInitialization, TraceEventType.Information);

            while (true)
            {
                Console.WriteLine(Resources.EnterNewGuessUIMessage);

                string guessString = Console.ReadLine();
                Program.Log(String.Format(CultureInfo.InvariantCulture, Resources.NewGuessLogMessage, guessString), LogEventIds.NewGuessReceived, TraceEventType.Verbose);

                try
                {
                    int guess = Int32.Parse(guessString, CultureInfo.CurrentCulture);

                    GuessResult result = Game.Guess(guess);

                    switch (result)
                    {
                        case GuessResult.TooLow:
                            Console.WriteLine(Resources.TooLowUIMessage);
                            break;
                        case GuessResult.TooHigh:
                            Console.WriteLine(Resources.TooHighUIMessage);
                            break;
                        case GuessResult.Success:
                            Console.WriteLine(Resources.WinUIMessage);
                            Game.NewRound();
                            break;                            
                        case GuessResult.WrongInput:
                            Console.WriteLine(Resources.WrongInputUIMessage);
                            break;
                        default:
                            break;
                    }
                }
                catch (Exception)
                {
                    string message = String.Format(CultureInfo.InvariantCulture, Resources.InvalidGuessLogMessage, guessString);
                    Program.Log(message, LogEventIds.InvalidGuess, TraceEventType.Warning);
                    Console.WriteLine(Resources.InvalidNumberUIMessage);                    
                }
            }
        }

        /// <summary>
        /// Helper method for logging with LogWriter
        /// </summary>
        /// <param name="message">Log entry's text</param>
        /// <param name="category">The category if the log entry</param>
        /// <param name="eventId">Unique Id of the log entry</param>
        /// <param name="severity">Severity of the log entry</param>
        internal static void Log(string message, string category, int eventId, TraceEventType severity)
        {
            if (Program.Logger.IsLoggingEnabled())
            {
                Program.Logger.Write(message, category, 0, eventId, severity);
            }
        }

        /// <summary>
        /// Helper method for logging with LogWriter
        /// </summary>
        /// <param name="message">Log entry's text</param>        
        /// <param name="eventId">Unique Id of the log entry</param>
        /// <param name="severity">Severity of the log entry</param>
        internal static void Log(string message, int eventId, TraceEventType severity)
        {
            Program.Log(message, LogCategories.Program, eventId, severity);
        }
    }    
}
